# Example usage of the supplied R functions to monitor systems of cointegrating
# regressions

# Author
# Etienne Theising
generateData = function(N = 3, Te = 1000, 
                        mu = 3, gamma = 1, beta1 = 1, beta2 = 1, 
                        rho1 = 0.3, rho2 = rho1, Sigma = diag(2 * N),
                        r = 1, nbreaks = 0, beta11 = beta1, beta21 = beta2
                        ) {
  require("matrixStats")
  require("MASS")
  nI1 = 0
  nSlope = 0
  if (nbreaks == 0) r = 1
  if (beta11 == beta1 & beta21 == beta2) nI1 = nbreaks
  else nSlope = nbreaks
  r.idx = floor(r * Te)
  eps = replicate(N, rnorm(Te))
  e = mvrnorm(n = Te + 2, mu = rep(0, 2 * N), Sigma = Sigma)
  e1 = e[, seq(1, 2 * N - 1, by = 2)]
  e2 = e[, seq(2, 2 * N, by = 2)]
  v1 = matrix(e1[2:(Te + 1), ] + 0.5 * e1[1:Te, ], nrow = Te, ncol = N)
  v2 = matrix(e2[2:(Te + 1), ] + 0.5 * e2[1:Te, ], nrow = Te, ncol = N)
  x1 = colCumsums(v1)
  x2 = colCumsums(v2)
  loopSeq = function(from, to) if (from <= to) return(from:to) else NULL
  u = matrix(0, nrow = Te, ncol = N)
  u[1, ] = eps[1, ] + rho2 * (e1[2, ] + e2[2, ])
  for (te in loopSeq(2, r.idx)) {
    u[te, ] = rho1 * u[te - 1, ] + eps[te, ] + 
      rho2 * (e1[te + 1, ] + e2[te + 1, ])
  }
  l.idx = loopSeq(r.idx + 1, Te)
  for (te in l.idx) {
    u[te, ] = c(rep(1, nI1), rep(rho1, N - nI1)) * u[te - 1, ] + eps[te, ] + 
      rho2 * (e1[te + 1, ] + e2[te + 1, ])
  }
  b.idx = loopSeq(1, nSlope)
  y = mu + gamma * matrix(1:Te, nrow = Te, ncol = N) +   
    beta1 * x1 + beta2 * x2 + u
  if (nSlope > 0 & r < 1) {
    y[l.idx, b.idx] = mu + 
      gamma * matrix(l.idx, nrow = Te - r.idx, ncol = nSlope) + 
      beta11 * x1[l.idx, b.idx] + beta21 * x2[l.idx, b.idx] + u[l.idx, b.idx]
  }
  x = kronecker(x1, matrix(1:0, ncol = 2)) + kronecker(x2, matrix(0:1, ncol = 2))
  return(list(y = y, x = x))
}
# Generate some data
test = generateData(nbreaks = 1, r = 0.25)
str(test)
# Monitor under homogeneity and independence assumption
mon = monitorCointSys(y = test$y, x = test$x, m = 0.25, trend = TRUE, 
                bandwidth = "and", kernel = "ba")
# Check results
str(mon)
# Monitor with arbitrary error structure
monGLS = monitorCointSysGLS(y = test$y, x = test$x, m = 0.25, trend = TRUE,
                          bandwidth = "and", kernel = "ba", simpl.var = TRUE)
# Check results
str(monGLS)